@@ -1,6 +1,7 @@
 import os
 import time
 import glob
+import datetime
 from pathlib import Path
 import re
 import logging
@@ -14,11 +15,14 @@ from pprint import pformat
 
 logger = logging.getLogger(__name__)
 
+CLEANUP_INTERVAL_MS = 0.999
+
 
 class BatchProcessor:
     def __init__(self, config):
         self.config = config
         self.do_log = self.config.get("batch-log-logging", True)
+        self.last_cleanup_time = datetime.datetime.now()
         # How many ids to batch simultaneously
         # self.batch_size=self.config.get('batch-log-batch-size', 10)
         # self.item_types=['scrape']
@@ -316,7 +320,7 @@ class BatchProcessor:
         # logger.info(f"No pending items found")
         return False
 
-    def insert_batch_item(self, type="test", data=None, result=None, priority=50, source=None, throttle_key=None, throttle_limit=1, throttle_period=1):
+    def insert_batch_item(self, type="test", data=None, ttl_seconds=None, result=None, priority=50, source=None, throttle_key=None, throttle_limit=1, throttle_period=1):
         """
         Insert a new batch item into the database, ready for execution
         """
@@ -324,6 +328,7 @@ class BatchProcessor:
         return self.db.insert_batch_item(
             {
                 "priority": priority,
+                "ttl_seconds": ttl_seconds, 
                 "data": data,
                 "result": result,
                 "type": type,
@@ -336,6 +341,9 @@ class BatchProcessor:
         )
         # fmt: on
 
+    def update_hang_status(self):
+        self.db.update_batch_hang_status()
+
     def retry_hung_jobs_older_than(self, time_sec=30):
         self.db.bump_batch_items(self.db.IN_PROGRESS, self.db.PENDING, time_sec)
 
@@ -362,7 +370,7 @@ class BatchProcessor:
             t = row.get("type")
             s = row.get("status")
             c = row.get("count")
-            totals[s] += count
+            totals[s] = totals.get(s, 0) + c
             out[t] = out.get(t, {})
             out[t][s] = c
         out["total"] = totals
@@ -375,12 +383,18 @@ class BatchProcessor:
             "last": self.db.get_last_jobs(limit=20),
             "last_failed": self.db.get_last_jobs(limit=20, status=["failed"]),
             "last_pending": self.db.get_last_jobs(limit=20, status=["pending"]),
-            "last_in_progress": self.db.get_last_jobs(limit=20, status=["in_progress"]),
+            "last_in_progress": self.db.get_last_jobs(limit=20, status=["in-progress"]),
             "counts": self.get_job_counts()}
         # fmt: on
         return status
 
     def process(self):
+        # Perform some cleanup on occation
+        now = datetime.datetime.now()
+        if now - self.last_cleanup_time > datetime.timedelta(seconds=CLEANUP_INTERVAL_MS):
+            self.last_cleanup_time = now
+            self.update_hang_status()
+        # Try to process one item
         if not self.execute_one_throttled_batch_item():
             # Put a status
             # if not self.last_status_time or (datetime.now() - self.last_status_time).total_seconds() > 10.0:
