@@ -7,6 +7,7 @@ class Database:
 
     PENDING = "pending"
     IN_PROGRESS = "in-progress"
+    HUNG = "hung"
     DONE = "done"
     FAILED = "failed"
 
@@ -24,6 +25,7 @@ class Database:
             create table if not exists "batch_log" (
                 id serial primary key,
                 priority int not null default 50,
+                ttl_seconds int, 
                 data text,
                 result text,
                 type varchar(255),
@@ -37,6 +39,8 @@ class Database:
                 updated_at timestamptz not null default now()
             );
             comment on column batch_log.id is 'Unique internal id for this batch item';
+            comment on column batch_log.priority is 'The batch item''s priority. Tasks with a higher number for priority will be picked first by executors. Tasks with the same number are equally important.';
+            comment on column batch_log.ttl_seconds is 'The batch item''s ttl (time to live) in seconds. When a job has spent more time in the state "in progress" than it''s TTL, it will be transitioned to the "hung" state indicating that it should be ignored until it has been expected by operator.';
             comment on column batch_log.data is 'The batch item''s data. Depends entirely on the type. Could for example be the URL to scrape for a site_scrape item';
             comment on column batch_log.result is 'The batch item''s result data. Depends entirely on the type. Could for example be the HTML scraped for the input URL for a site_scrape item';
             comment on column batch_log.type is 'The batch item''s type such as order_scrape or site_scrape';
@@ -125,6 +129,7 @@ $$ language plpgsql;
                 insert into batch_log
                     (
                     priority,
+                    ttl_seconds,
                     data,
                     result,
                     type,
@@ -137,6 +142,7 @@ $$ language plpgsql;
                 values
                     (
                     %(priority)s,
+                    %(ttl_seconds)s,
                     %(data)s,
                     %(result)s,
                     %(type)s,
@@ -149,6 +155,7 @@ $$ language plpgsql;
                 on conflict(id)
                 do update
                     set priority=%(priority)s,
+                    ttl_seconds=%(ttl_seconds)s,
                     data=%(data)s,
                     result=%(result)s,
                     type=%(type)s,
@@ -171,6 +178,7 @@ $$ language plpgsql;
             """
                 update batch_log
                     set priority=%(priority)s,
+                    ttl_seconds=%(ttl_seconds)s,
                     data=%(data)s,
                     result=%(result)s,
                     type=%(type)s,
@@ -189,6 +197,23 @@ $$ language plpgsql;
             batch_item,
         )
 
+    # Update status of all "in progress" jobs that already spent more than ttl time to "hung"
+    def update_batch_hang_status(self):
+        return self.dbc.query_none(
+            """
+                update batch_log
+                    set
+                    status=%(to_status)s,
+                    updated_at=now()
+                where status = %(from_status)s
+                and ttl_seconds is not null
+                and ttl_seconds > 0
+                and updated_at < ( now() - (ttl_seconds * interval '1 second'))
+                ;
+                """,
+            {"from_status": (IN_PROGRESS,), "to_status": (HUNG,)},
+        )
+
     # Update status of all jobs with from_status to to_status given they have time delta longer than given time
     def bump_batch_items(self, from_status=IN_PROGRESS, to_status=PENDING, time_sec=30):
         return self.dbc.query_none(
@@ -301,7 +326,7 @@ $$ language plpgsql;
         self.dbc.query_none(
             """
                 delete from batch_log
-                where status=%(status)s,
+                where status = %(status)s,
                 and updated_at >= %(updated_at)s
                 ;
                 """,
@@ -318,12 +343,13 @@ $$ language plpgsql;
         )
 
     # Get batch items from batch log fitting the optional filter
-    def get_batch_items(self, id=None, priority=None, type=None, status=None, throttle_key=None, source=None, limit=1):
+    def get_batch_items(self, id=None, priority=None, ttl_seconds=None, type=None, status=None, throttle_key=None, source=None, limit=1):
         return self.dbc.query_many(
             """
                 select
                     id,
                     priority,
+                    ttl_seconds,
                     data,
                     result,
                     type,
@@ -338,6 +364,7 @@ $$ language plpgsql;
                 where true
                 and (%(id)s is null or id = any(%(id)s))
                 and (%(priority)s is null or priority = any(%(priority)s))
+                and (%(ttl_seconds)s is null or ttl_seconds = any(%(ttl_seconds)s))
                 and (%(type)s is null or type = any(%(type)s))
                 and (%(status)s is null or status = any(%(status)s))
                 and (%(throttle_key)s is null or throttle_key = any(%(throttle_key)s))
@@ -356,6 +383,7 @@ $$ language plpgsql;
                 select
                     id,
                     priority,
+                    ttl_seconds,
                     data,
                     result,
                     type,
@@ -466,6 +494,7 @@ $$ language plpgsql;
                 select
                     id,
                     priority,
+                    ttl_seconds, 
                     data,
                     result,
                     type,
